import json
import os
from pathlib import Path

from PyQt6.QtGui import QIcon

CURRENT_DIRECTORY = os.path.dirname(os.path.realpath(__file__))


def _load_images():
    return {
        'icon': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "icon.png")),
        'notebook': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "notebook--pencil.png")),
        'document': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "blue-document--plus.png")),
        'arrow': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "arrow-turn-000-left.png")),
        'cross': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "cross.png")),
        'swap': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "swap.png")),
        'exit': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "exit.png")),
        'window': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "window.png")),
        'highlight': QIcon(os.path.join(CURRENT_DIRECTORY, "images", "highlighter-text.png")),
    }


class Config:
    def __init__(self):
        self.images = _load_images()
        self.system_config = self._load_config()

    def _load_config(self):  # noqa
        path = Path.home() / ".kleantrans/config.json"
        with open(path, encoding='utf-8') as rd:
            config = json.load(rd)
            return config

    def save_config(self):
        path = Path.home() / ".kleantrans/config.json"
        with open(path, 'w', encoding='utf-8') as wt:
            json.dump(self.system_config, wt)

    @property
    def source_lang(self):
        return self.system_config["swap"][0]

    @property
    def target_lang(self):
        return self.system_config["swap"][1]

    @property
    def active_capture_text(self):
        return self.system_config["active"]



import os
import pyperclip

from PyQt6.QtCore import pyqtSignal, QObject

from pynput import mouse

from .translator import Translator, TranslatedText
from .config import Config


class MouseListener(QObject):
    raw_text_signal = pyqtSignal(TranslatedText)

    def __init__(self, translator: Translator, system_config: Config):
        super().__init__()
        self.translator = translator
        self.system_config = system_config
        self.mouse = mouse.Listener(on_click=self._on_click)
        self.mouse.start()

    def _on_click(self, x, y, button: mouse.Button, pressed: bool):  # noqa
        if self.system_config.active_capture_text and button == mouse.Button.middle and pressed:
            os.system("xclip -out -selection primary | xclip -in -selection clipboard")

            text = self.translator.clean_text(pyperclip.paste())
            trans_text = self.translator.translate(text, self.system_config.source_lang, self.system_config.target_lang)

            self.raw_text_signal.emit(TranslatedText(text, trans_text))  # noqa



import os
import pyperclip

from PyQt6.QtCore import pyqtSignal, QObject
from pynput import keyboard

from .translator import Translator, TranslatedText
from .config import Config


class KeyboardListener(QObject):
    raw_text_signal = pyqtSignal(TranslatedText)
    hide_window_signal = pyqtSignal(bool)

    def __init__(self, translator: Translator, system_config: Config):
        super().__init__()
        self.translator = translator
        self.system_config = system_config
        self.switch = True

        self.window_combination = keyboard.HotKey(
            keyboard.HotKey.parse("<shift>+<cmd>"),
            self._on_active)
        self.translate_combination = keyboard.HotKey(
            keyboard.HotKey.parse("<ctrl>+<cmd>"),
            self._on_translate)

        self.window_combination_listener = keyboard.Listener(
            on_press=self._for_canonical(self.window_combination.press),
            on_release=self._for_canonical(self.window_combination.release))
        self.window_combination_listener.start()

        self.translate_combination_listener = keyboard.Listener(
            on_press=self._for_canonical(self.translate_combination.press),
            on_release=self._for_canonical(self.translate_combination.release))
        self.translate_combination_listener.start()

    def _on_active(self):
        if self.switch:
            self.hide_window_signal.emit(True)
        else:
            self.hide_window_signal.emit(False)

        self.switch = not self.switch

    def _on_translate(self):
        # If the screen is show and user turn on translating from capture text fearure
        if self.switch and self.system_config.active_capture_text:
            os.system("xclip -out -selection primary | xclip -in -selection clipboard")

            text = self.translator.clean_text(pyperclip.paste())
            trans_text = self.translator.translate(text, self.system_config.source_lang, self.system_config.target_lang)

            # Send signal to main_window.py
            self.raw_text_signal.emit(TranslatedText(text, trans_text))

    def _for_canonical(self, f):
        return lambda k: f(self.window_combination_listener.canonical(k))



from PyQt6.QtWidgets import \
    QDialog, QDialogButtonBox, QVBoxLayout, QLabel


class Dialog:
    def __init__(self, title, text):
        self._title = title
        self._text = text

    @property
    def title(self):
        return self._title

    @property
    def text(self):
        return self._text


class CustomDialog(QDialog):
    def __init__(self):
        super().__init__()
        self.dialog = None

        self.setWindowTitle("")
        self.buttonBox = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.accepted.connect(self.accept)
        self.layout = QVBoxLayout()
        self.message = QLabel("")
        self.layout.addWidget(self.message)
        self.layout.addWidget(self.buttonBox)
        self.setLayout(self.layout)

    def set_message(self, dialog: Dialog):
        self.setWindowTitle(dialog.title)
        self.message.setText(dialog.text)



import subprocess
import re

from PyQt6.QtCore import pyqtSignal, QObject

from .dialog import Dialog


class TranslatedText:
    def __init__(self, raw_text: str, translated_text: str):
        self._raw_text = raw_text
        self._translated_text = translated_text

    @property
    def raw_text(self):
        return self._raw_text

    @property
    def translated_text(self):
        return self._translated_text


class Translator(QObject):
    dialog_signal = pyqtSignal(Dialog)

    def __init__(self):
        super().__init__()
        self.pattern = r"[^a-zA-Z0-9?!'()\.,+-=:;\s]"

    def translate(self, text: str, source: str, dest: str) -> str:  # noqa
        trans_text = ""

        if text != "":
            try:
                cmd = """
                    trans -e google -s {} -t {} -show-original y -show-original-phonetics n -show-translation y -no-ansi -show-translation-phonetics n -show-prompt-message n -show-languages y -show-original-dictionary n -show-dictionary n -show-alternatives n "{}"
                    """.format(source, dest, text)

                subprocess_ = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
                trans_text = subprocess_.stdout.read().decode('utf-8').split("\n\n")[1]
                trans_text = re.sub("(u200b)", "", trans_text)
            except (Exception,) as e:
                self.dialog_signal.emit(Dialog("Translation Error", str(e)))

        return trans_text

    def clean_text(self, text):
        text = re.sub(r"\s+", " ", re.sub(self.pattern, " ", text.strip())).strip()
        return text

    def ignore_clean_this_lang(self, lang: str):
        if lang in ("vi",):
            return True

        return False



from PyQt6.QtWidgets import \
    QMainWindow, QCheckBox, QToolBar, QLabel, QStatusBar, QVBoxLayout, \
    QPlainTextEdit, QWidget, QMenu, QSystemTrayIcon, QApplication
from PyQt6.QtCore import Qt, QSize
from PyQt6.QtGui import QAction, QKeySequence, QShortcut

from .config import Config
from .mouse_listener import MouseListener
from .keyboard_listener import KeyboardListener
from .dialog import Dialog, CustomDialog
from .translator import Translator, TranslatedText


class MainWindow(QMainWindow):
    def __init__(self, app: QApplication, config: Config, translator: Translator):
        super().__init__()
        self.app = app
        self.config = config
        self.translator = translator
        self.raw_text = None
        self.for_text = None
        self.window_pos = (self.pos().x(), self.pos().y())

        self.setWindowIcon(self.images['icon'])
        self.setWindowTitle("KleanTrans")
        self.setGeometry(660, 340, 600, 400)
        self.setWindowFlags(Qt.WindowType.WindowStaysOnTopHint)  # Always on top

        self._setup_listeners()
        self._setup_toolbar()
        self._setup_widgets()
        self._setup_dialogs()
        self._setup_hotkeys()
        self._setup_tray()

        self._setup_post_main_window()

    def _setup_listeners(self):
        def set_raw_text(translated_text: TranslatedText):
            if self.raw_text is not None:
                self.raw_text.setPlainText(translated_text.raw_text)

            if self.for_text is not None:
                self.for_text.setPlainText(translated_text.translated_text)

        def hide_window(show: bool):
            if show:
                self.move(self.window_pos[0], self.window_pos[1])
                self.show()
            else:
                self.window_pos = (self.pos().x(), self.pos().y())
                self.hide()

        self.mouse_listener = MouseListener(self.translator, self.config)
        self.mouse_listener.raw_text_signal.connect(set_raw_text)

        self.keyboard_listener = KeyboardListener(self.translator, self.config)
        self.keyboard_listener.raw_text_signal.connect(set_raw_text)
        self.keyboard_listener.hide_window_signal.connect(hide_window)

    def _setup_toolbar(self):
        btn_config = QAction(self.images['notebook'], 'Configure', self)
        btn_config.setStatusTip('Configure the characters used to clean text.')
        # btn_config.triggered.connect(self.show_config_window)

        toolbar = QToolBar()
        toolbar.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        toolbar.setIconSize(QSize(16, 16))

        self._configure_capture_checkbox(toolbar)
        self._configure_swap_button(toolbar)
        self._configure_checkbox_hide(toolbar)
        toolbar.addWidget(QLabel("Copyright © by Cuong. Duong Manh"))

        self.addToolBar(toolbar)
        self.setStatusBar(QStatusBar(self))

    def _configure_capture_checkbox(self, toolbar: QToolBar):
        def connect_capture_checkbox(checked: bool):
            self._switch_capture_feature()
            self.config.save_config()

        self.chb_active = QCheckBox('Capture')
        self.chb_active.setStatusTip('Press Ctrl+T to turn ON/OFF translating from text capture.')
        self.chb_active.clicked.connect(connect_capture_checkbox)  # noqa
        self.chb_active.setChecked(self._capture_text_appear())

        # Add this widget into the toolbar layout
        toolbar.addWidget(self.chb_active)
        toolbar.addSeparator()

    def _configure_swap_button(self, toolbar: QToolBar):
        def connect_swap_button():
            self.system_config['swap'] = [self.system_config['swap'][1], self.system_config['swap'][0]]
            self.btn_swap.setText(f"{self.system_config['swap'][0]} ➜ {self.system_config['swap'][1]}")
            self.raw_text.setPlainText("")
            self.config.save_config()

        self.btn_swap = QAction(
            self.images['swap'],
            f"{self.system_config['swap'][0]} ➜ {self.system_config['swap'][1]}", self)

        self.btn_swap.setStatusTip('Press Ctrl+W to swap two languages.')
        self.btn_swap.setShortcut(QKeySequence('Ctrl+w'))

        # Setup trigger
        self.btn_swap.triggered.connect(connect_swap_button)  # noqa

        # Add this widget into the toolbar layout
        toolbar.addAction(self.btn_swap)
        toolbar.addSeparator()

    def _configure_checkbox_hide(self, toolbar: QToolBar):
        def connect_hide_checkbox(checked):
            self._hide_raw_text(checked)
            self._main_window_switch()
            self.config.save_config()

        self.chb_hide = QCheckBox('Hide')
        self.chb_hide.setStatusTip('Hide the top text box.')
        self.chb_hide.clicked.connect(connect_hide_checkbox)  # noqa
        self.chb_hide.setChecked(self._main_window_appear())

        toolbar.addWidget(self.chb_hide)
        toolbar.addSeparator()

    def _setup_hotkeys(self):
        def translate():
            clean_text = self.raw_text.toPlainText().strip()
            if not self.translator.ignore_clean_this_lang(self.config.source_lang):
                clean_text = self.translator.clean_text(clean_text)

            translated_text = self.translator.translate(clean_text, self.config.source_lang,
                                                        self.config.target_lang)
            self.raw_text.setPlainText(clean_text)
            self.for_text.setPlainText(translated_text)

        def clear():
            self.raw_text.setPlainText('')
            self.for_text.setPlainText('')

        def active():
            self._switch_capture_feature()
            self.chb_active.setChecked(self._capture_text_appear())

        btn_trans_shortcut = QShortcut(QKeySequence('Ctrl+Return'), self)
        btn_trans_shortcut.activated.connect(translate)  # noqa

        btn_clear_shortcut = QShortcut(QKeySequence('Ctrl+d'), self)
        btn_clear_shortcut.activated.connect(clear)  # noqa

        chb_active_shortcut = QShortcut(QKeySequence('Ctrl+t'), self)
        chb_active_shortcut.activated.connect(active)  # noqa

    def _setup_tray(self):
        def connect_capture_checkbox():
            self._switch_capture_feature()

            capture_text_inside = "Enable Capture"
            if self.config.active_capture_text:
                capture_text_inside = "Disable Capture"

            self.capture_action.setText(capture_text_inside)
            self.chb_active.setChecked(self._capture_text_appear())
            self.config.save_config()

        capture_text = "Enable Capture"
        if self.config.active_capture_text:
            capture_text = "Disable Capture"

        self.show_action = QAction(self.images['window'], 'Show')
        self.show_action.triggered.connect(self.show)  # noqa
        self.quit_action = QAction(self.images['exit'], 'Quit')
        self.quit_action.triggered.connect(self.app.quit)  # noqa
        self.capture_action = QAction(self.images['highlight'], capture_text)
        self.capture_action.triggered.connect(connect_capture_checkbox)  # noqa

        self.menu = QMenu()
        self.menu.addAction(self.show_action)
        self.menu.addAction(self.capture_action)
        self.menu.addAction(self.quit_action)

        self.tray = QSystemTrayIcon()
        self.tray.setIcon(self.images['icon'])
        self.tray.setVisible(True)
        self.tray.setContextMenu(self.menu)
        self.tray.show()

    def _setup_widgets(self):
        layout = QVBoxLayout()

        self.raw_text = QPlainTextEdit()
        layout.addWidget(self.raw_text)

        self.for_text = QPlainTextEdit()
        layout.addWidget(self.for_text)

        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)

    def _setup_dialogs(self):
        def show_dialog(dialog: Dialog):
            self.dialog.set_message(dialog)
            self.dialog.setFixedSize(250, 70)
            self.dialog.exec()

        self.dialog = CustomDialog()
        self.translator.dialog_signal.connect(show_dialog)

    def _setup_post_main_window(self):
        self._hide_raw_text(self.system_config["hide"])

    @property
    def images(self):
        return self.config.images

    @property
    def system_config(self):
        return self.config.system_config

    def _main_window_appear(self) -> bool:
        return self.system_config["hide"]

    def _main_window_switch(self):
        self.system_config["hide"] = not self.system_config["hide"]

    def _capture_text_appear(self) -> bool:
        return self.system_config["active"]

    def _switch_capture_feature(self):
        self.system_config["active"] = not self.system_config["active"]

    def _hide_raw_text(self, checked: bool):
        if self.raw_text is not None:
            if checked:
                self.resize(600, 250)
                self.raw_text.hide()
            else:
                self.resize(600, 400)
                self.raw_text.show()



from setuptools import setup, find_packages

setup(
    name="kleantrans",
    version="1.0.6",
    packages=find_packages(),
    include_package_data=True,  # Include data files specified in MANIFEST.in
    package_data={
        '': ['app/images/*'],  # Include all files from the images folder
    },
)



import sys
import os

from PyQt6.QtWidgets import QApplication

from .main_window import MainWindow
from .translator import Translator
from .config import Config


def _init_kleantrans_directory():
    import json
    from pathlib import Path

    # Get the home directory of the user
    home_dir = Path.home()

    # Define the path for the .kleantrans directory
    kleantrans_dir = home_dir / ".kleantrans"

    # Check if the directory exists, and if not, create it
    if not kleantrans_dir.exists():
        kleantrans_dir.mkdir(parents=True)
        path = f"{kleantrans_dir}/config.json"
        data = {
            "swap": ["en", "vi"],
            "active": True,
            "hide": False
        }
        with open(path, 'w', encoding='utf-8') as wt:
            json.dump(data, wt)


def run():
    try:
        # First fork
        pid = os.fork()
        if pid > 0:
            sys.exit(0)  # Exit first parent

    except OSError as e:
        sys.stderr.write(f"Fork #1 failed: {e.strerror}\n")
        sys.exit(1)

    # Decouple from parent environment
    os.chdir("/")
    os.setsid()
    os.umask(0)

    try:
        # Second fork
        pid = os.fork()
        if pid > 0:
            sys.exit(0)  # Exit second parent

    except OSError as e:
        sys.stderr.write(f"Fork #2 failed: {e.strerror}\n")
        sys.exit(1)

    _init_kleantrans_directory()

    translator = Translator()

    app = QApplication(sys.argv)
    app.setQuitOnLastWindowClosed(False)
    app.setStyle('Fusion')

    config = Config()
    MainWindow(app, config, translator)

    app.exec()

# if __name__ == '__main__':
#     sys.exit(run())
